import wx
from SimulatorFramework import *
from SimulatorFramework.TileType import *
from PIL.ImageDraw2 import Pen


N = North = 0
E = East = 1
S = South = 2
W = West = 3
U = Up = 4
D = Down = 5

Dirs = [N,E,S,W]
DirAbbrevArray = ("N", "E", "S", "W", "U", "D")
DirNameArray = ("NORTH", "EAST", "SOUTH", "WEST", "UP", "DOWN")

def GetFontSizeToFitTextInBounds(text, font, dc, width, height):
    retFont = font
    (w, pixelSize) = retFont.GetPixelSize()
    bContinue = True
    while bContinue == True:
        dc.SetFont(retFont)
        (w, h) = dc.GetTextExtent(text)
        if ((w <= width) and (h <= height)) or pixelSize <= 2:
            bContinue = False
        else:
            pixelSize = pixelSize - 2
            retFont.SetPixelSize((0, pixelSize))
    #(w, pixelSize) = retFont.GetPixelSize()
    #print("pixelSize: " + str(pixelSize))
    return retFont


CELLSIZE = 60
CELLSIZE = 60

# for 3D simulation
#----------------------------------------------------------
# in OpenGL 1.0 texture sides should be 2^n
TEXTURE_RES = 1024

# size of a side of a tile on the texture
SIDE_RES = int(TEXTURE_RES / 3)

# texture offsets of given side [N,E,S,W,U,D]
SIDE_OFFSET = [(0,0),(0,SIDE_RES),(SIDE_RES,0),(SIDE_RES,SIDE_RES),(2*SIDE_RES,0),(2*SIDE_RES,SIDE_RES)]
#----------------------------------------------------------

class PyTASTileType(TileType):
    """A representation of a tile type for use in the ISU TAS simulator.
       Every tile must have a unique name and a unique set of 4 glues. If
       a name is not explicitly provided, one will be formed automatically
       as a concatenation of the glues values of each side."""
    
    def __init__(self, name="", width=CELLSIZE, height=CELLSIZE, is3D=False):
        TileType.__init__(self, name, width, height)
        self.label = ""
        self.color = "White"
        self.colorVector = (255, 255, 255)
        self.textColor = "Black"
        self.textColorVector = (0, 0, 0)
        self.img = None
        if is3D == False:
            self.dims = 2
            self.sides = {N:Glue(),E:Glue(),S:Glue(),W:Glue()}
        else:
            self.dims = 3
            self.sides = {N:Glue(),E:Glue(),S:Glue(),W:Glue(),U:Glue(),D:Glue()}
           
    def SetDimensions(self, dims):
        if self.dims == dims:
            return
        elif dims == 3:
            self.dims = 3
            self.sides[U] = Glue()
            self.sides[D] = Glue()
        elif dims == 2:
            self.dims = 2
            savedSides = self.sides
            self.sides = {N:savedSides[N], E:savedSides[E], S:savedSides[S], W:savedSides[W]}
        else:
            print("ERROR: invalid number of tile type dimensions: " + str(dims))

    def DrawAtLocation(self, xPixel, yPixel, pdc):
        if self.img is None:
            self.MakeImage()
        #pdc.DrawBitmap(self.img, xPixel, yPixel)
        savedScale = pdc.GetUserScale()
        adj = savedScale[0]/3
        pdc.SetUserScale(adj, adj)
        pdc.DrawBitmap(self.img, xPixel*3, yPixel*3)
        pdc.SetUserScale(savedScale[0], savedScale[1])

    def MakeImage(self, backgroundColor=None):
            w = float(CELLSIZE*3)
            h = float(CELLSIZE*3)
            bmp = wx.Bitmap(w, h)
            dc = wx.MemoryDC()
            dc.SelectObject(bmp)
            
            if not backgroundColor is None:
                bg = wx.Brush(backgroundColor)#(self.GetBackgroundColour())
                dc.SetBackground(bg)
            dc.Clear()

            brush = wx.Brush(self.colorVector)
            dc.SetBrush(brush)
            
            black = wx.TheColourDatabase.Find("Black")
            pen = wx.Pen(black, width=3)
            dc.SetPen(pen)

            tabSize = 10.0
            dc.DrawRectangle(0+tabSize, 0+tabSize, w-(2*tabSize), h-(2*tabSize))
            
            brush = wx.Brush(black)
            dc.SetBrush(brush)

            # Draw northern tabs
            numTabs = self.sides[N].strength
            if numTabs > 0:
                spacing = w / (numTabs+1)
                currLoc = spacing
                for n in range(numTabs):
                    dc.DrawRectangle(currLoc-(tabSize/2), 0, tabSize, tabSize)
                    currLoc = currLoc + spacing

            # Draw southern tabs
            numTabs = self.sides[S].strength
            if numTabs > 0:
                spacing = w / (numTabs+1)
                currLoc = spacing
                for n in range(numTabs):
                    dc.DrawRectangle(currLoc-(tabSize/2), h-tabSize, tabSize, tabSize)
                    currLoc = currLoc + spacing
            
            # Draw western tabs
            numTabs = self.sides[W].strength
            if numTabs > 0:
                spacing = h / (numTabs+1)
                currLoc = spacing
                for n in range(numTabs):
                    dc.DrawRectangle(0, currLoc-(tabSize/2), tabSize, tabSize)
                    currLoc = currLoc + spacing

            # Draw eastern tabs
            numTabs = self.sides[E].strength
            if numTabs > 0:
                spacing = h / (numTabs+1)
                currLoc = spacing
                for n in range(numTabs):
                    dc.DrawRectangle(w-tabSize, currLoc-(tabSize/2), tabSize, tabSize)
                    currLoc = currLoc + spacing

            dc.SetTextForeground(self.textColorVector)

            # Draw tile label
            wLabel = (w - 2*tabSize)/2
            hLabel = (h - 2*tabSize)/2
            defPixHt = 30
            weight = wx.FONTWEIGHT_BOLD
            ftype = wx.FONTFAMILY_DEFAULT#wx.FONTFAMILY_MODERN
            glueLabelScale = 1.0
            
            fname = ""#"Arial Narrow"
            #font = wx.Font(pointSize = 30, family = wx.DEFAULT, style = wx.NORMAL, weight = wx.NORMAL, faceName = 'Arial')
            if not self.label is None:
                font = wx.Font(pixelSize = (0,defPixHt), family = ftype, style = wx.NORMAL, weight = weight, faceName = fname)#, faceName = 'Arial')
                font = GetFontSizeToFitTextInBounds(self.label, font, dc, wLabel, hLabel)
                (wLabel, hLabel) = dc.GetTextExtent(self.label)
                dc.DrawText(self.label, (w/2)-(wLabel/2), (h/2)-(hLabel/2))
            
            longestLabel = ""
            for d in Dirs:
                if len(self.sides[d].label) > len(longestLabel):
                    longestLabel = self.sides[d].label

            # Draw north glue label
            defWLabel = (w - 6*tabSize) * glueLabelScale
            defHLabel = (h - 6*tabSize) * glueLabelScale
            wLabel = defWLabel
            hLabel = defHLabel
            #font = wx.Font(pixelSize = (0,defPixHt), family = ftype, style = wx.NORMAL, weight = wx.FONTWEIGHT_NORMAL, faceName = fname)
            font = wx.Font(pointSize = 32, family = ftype, style = wx.NORMAL, weight = wx.FONTWEIGHT_NORMAL, faceName = fname)
            font = GetFontSizeToFitTextInBounds(longestLabel, font, dc, wLabel, hLabel)
            (wLabel, hLabel) = dc.GetTextExtent(self.sides[N].label)
            dc.DrawText(self.sides[N].label, (w/2)-(wLabel/2), 10)
            
            # Draw south glue label
            wLabel = defWLabel
            hLabel = defHLabel
            #font = wx.Font(pixelSize = (0,defPixHt), family = ftype, style = wx.NORMAL, weight = wx.FONTWEIGHT_NORMAL, faceName = fname)
            #font = GetFontSizeToFitTextInBounds(self.sides[S].label, font, dc, wLabel, hLabel)
            (wLabel, hLabel) = dc.GetTextExtent(self.sides[S].label)
            dc.DrawText(self.sides[S].label, (w/2)-(wLabel/2), h-(hLabel+12))
            
            # Draw west glue label
            wLabel = defWLabel
            hLabel = defHLabel
            #font = wx.Font(pixelSize = (0,defPixHt), family = ftype, style = wx.NORMAL, weight = wx.FONTWEIGHT_NORMAL, faceName = fname)
            #font = GetFontSizeToFitTextInBounds(self.sides[W].label, font, dc, wLabel, hLabel)
            (wLabel, hLabel) = dc.GetTextExtent(self.sides[W].label)
            dc.DrawRotatedText(self.sides[W].label, tabSize, (h/2)+(wLabel/2), 90.0)

            # Draw east glue label
            wLabel = defWLabel
            hLabel = defHLabel
            #font = wx.Font(pixelSize = (0,defPixHt), family = ftype, style = wx.NORMAL, weight = wx.FONTWEIGHT_NORMAL, faceName = fname)
            #font = GetFontSizeToFitTextInBounds(self.sides[E].label, font, dc, wLabel, hLabel)
            (wLabel, hLabel) = dc.GetTextExtent(self.sides[E].label)
            dc.DrawRotatedText(self.sides[E].label, w-tabSize, (h/2)-(wLabel/2), 270.0)

            dc.SelectObject(wx.NullBitmap)
            
            #img = bmp.ConvertToImage()
            #img = img.Scale(CELLSIZE, CELLSIZE, wx.IMAGE_QUALITY_HIGH)
            #bmp = wx.Bitmap(img)
            self.img = bmp

    def Copy(self):
        if self.dims == 2:
            t = PyTASTileType()
        else:
            t = TileType(True)
        t.sides[N] = self.sides[N].Copy()
        t.sides[E] = self.sides[E].Copy()
        t.sides[S] = self.sides[S].Copy()
        t.sides[W] = self.sides[W].Copy()
        if self.dims == 3:
            t.sides[U] = self.sides[U].Copy()
            t.sides[D] = self.sides[D].Copy()
        t.name = self.name
        t.label = self.label
        t.color = self.color
        t.textColor = self.textColor
        t.colorVector = self.colorVector
        t.textColorVector = self.textColorVector
        if self.img is None:
            t.img = None
        else:
            t.img = self.img.GetSubBitmap(wx.Rect(0, 0, self.img.GetWidth(), self.img.GetHeight()))
        return t
    
    def SetEqual(self, other):
        TileType.SetEqual(self, other)
        if other.img is None:
            self.img = None
        else:
            self.img = other.img.GetSubBitmap(wx.Rect(0, 0, other.img.GetWidth(), other.img.GetHeight()))

    def SetInputSides(self, inputSet):
        for k in inputSet:
            self.sides[k] = Glue(inputSet[k].strength, inputSet[k].label)


    # function to parse the color string into a color vector
    def ParseColor(self):
        if self.color.lower().startswith("rgba"):
            try:
                self.colorVector = eval(self.color[4:])
            except:
                self.colorVector = (255, 255, 255)
        else:
            col = wx.TheColourDatabase.Find(self.color)
            if col.IsOk():
                self.colorVector = (col.Red(), col.Green(), col.Blue())
            else:
                if not self.color.lower().startswith("rgb"):
                    self.colorVector = (255, 255, 255)
                else:
                    channels = self.color[4:-1].split(",")
                    try:
                        r = int(channels[0].strip())
                        g = int(channels[1].strip())
                        b = int(channels[2].strip())
                        self.colorVector = (r, g, b)
                    except:
                        #print "Color Error on Tile: {}".format(self.name)#DEBUG
                        self.colorVector = (255, 255, 255)

        if self.textColor.lower().startswith("rgba"):
            try:
                self.textColorVector = eval(self.textColor[4:])
            except:
                self.textColorVector = (255, 255, 255)
        else:
            col = wx.TheColourDatabase.Find(self.textColor)
            if col.IsOk():
                self.textColorVector = (col.Red(), col.Green(), col.Blue())
            else:
                if not self.textColor.lower().startswith("rgb"):
                    self.textColorVector = (255, 255, 255)
                else:
                    channels = self.textColor[4:-1].split(",")
                    try:
                        r = int(channels[0].strip())
                        g = int(channels[1].strip())
                        b = int(channels[2].strip())
                        self.textColorVector = (r, g, b)
                    except:
                        #print "Color Error on Tile: {}".format(self.name)#DEBUG
                        self.textColorVector = (0, 0, 0)

#----------------------------------------------------------------------------
class NeighborGlues():
    """Description of glues which border a frontier location, derived from neighbor tiles"""
    name = ""
    sides = {}
    dims  = 2
    
    def __init__(self, name="", is3D=False):
        self.name = name
        if is3D == False:
            self.dims = 2
            self.sides = {N:Glue(),E:Glue(),S:Glue(),W:Glue()}
        else:
            self.dims = 3
            self.sides = {N:Glue(),E:Glue(),S:Glue(),W:Glue(),U:Glue(),D:Glue()}
           